Hooks:PostHook(BowWeaponBase, "init", "Hawkeye_BowWeaponBase_init", function(self, unit)
    self._bow_multishot_overload =  managers.player:has_category_upgrade("bow", "multishot_overload") and managers.player:upgrade_value("bow", "multishot_overload")
    -- log("self._bow_multishot_overload"..tostring(self._bow_multishot_overload))
    self._overcharged_enabled = managers.player:has_category_upgrade("bow", "overcharge")
    self._overcharged_shot = false
    if self._overcharged_enabled then
        self._overcharge_cooldown = managers.player:upgrade_value("bow", "overcharge").cd
        self._overcharge_charge_min_duration = managers.player:upgrade_value("bow", "overcharge").charge
        self._next_overcharge_at_time = 0
    end
    if self._bow_multishot_overload then
        DelayedCalls:Add("Set_bow_multishot_overload", 0.2, function()
            self:set_ammo_max_per_clip(self._bow_multishot_overload or 1)
        end)
    end
end)

function BowWeaponBase:charge_max_t()
    -- overrides original function, multiply by buff value        
    return self:weapon_tweak_data().charge_data.max_t / managers.player:upgrade_value("bow", "charge_speed_multiplier", 1)
end

function BowWeaponBase:manages_steelsight()
    -- return not managers.player:has_category_upgrade("bow", "hip_run_and_shoot")
    return false
end

-- Replaces old crappy function with a proper one
function BowWeaponBase:interupt_charging()
    if self._cancelled then
        return
    end

    if self._charging then
        self._cancelled = true
        self._overcharged_shot = false
        self:play_tweak_data_sound("charge_cancel")
    end
end

function BowWeaponBase:run_and_shoot_allowed()
    local allowed = BowWeaponBase.super.run_and_shoot_allowed(self)

    return allowed or managers.player:has_category_upgrade("bow", "hip_run_and_shoot")
end


function BowWeaponBase:fire(from_pos, direction, dmg_mul, shoot_player, spread_mul, autohit_mul, suppr_mul, target_unit)
    local rotation_factor_base = 5
    local rotation_factor_flip = true
    local rotation_factor_prev = 0
    if self._bow_multishot_overload and self:ammo_base():get_ammo_remaining_in_clip() > 1 then
        for i = 2, self:ammo_base():get_ammo_remaining_in_clip() do
            -- log("rotation_factor_base: "..tostring(rotation_factor_base))
            local rotation = rotation_factor_base * (rotation_factor_flip and 1 or -1)
            rotation_factor_flip = not rotation_factor_flip
            if self._overcharged_shot then -- Overcharge effect on multi shot factors
                rotation = rotation * 1.5
            end
            rotation_factor_base = math.abs(rotation_factor_prev) - math.abs(rotation) == 0 and (rotation_factor_base + 5) or rotation_factor_base
            rotation_factor_prev = rotation
            local new_direction = direction:rotate_with(Rotation(rotation, 0, 0))
            BowWeaponBase.super.fire(self, from_pos, new_direction, dmg_mul, shoot_player, spread_mul, autohit_mul,
                suppr_mul, target_unit)
        end
    end

    if self._overcharged_shot then
        self._next_overcharge_at_time = managers.player:player_timer():time() + self._overcharge_cooldown
        managers.player:set_ability_on_cooldown("HAWKEYE_Overcharge", self._overcharge_cooldown)
        DelayedCalls:Add("turn_off_overcharged_shot", 0.02, function()
            self._overcharged_shot = false
        end)
    end

    return BowWeaponBase.super.fire(self, from_pos, direction, dmg_mul, shoot_player, spread_mul, autohit_mul, suppr_mul, target_unit)
end

local HAWKEYE_BowWeaponBase_on_reload = BowWeaponBase.on_reload
function BowWeaponBase:on_reload(amount)
    if not self._bow_multishot_overload then
        HAWKEYE_BowWeaponBase_on_reload(self, amount)
    else
        amount = math.min(self:ammo_base():get_ammo_remaining_in_clip() + 1, self:ammo_base():get_ammo_max_per_clip())
        local ammo_base = self._reload_ammo_base or self:ammo_base()
        amount = amount or ammo_base:get_ammo_max_per_clip()
        if self._setup.expend_ammo then
            ammo_base:set_ammo_remaining_in_clip(math.min(ammo_base:get_ammo_total(), amount))
        else
            ammo_base:set_ammo_remaining_in_clip(amount)
            ammo_base:set_ammo_total(amount)
        end
        managers.job:set_memory("kill_count_no_reload_" .. tostring(self._name_id), nil, true)
        self._reload_ammo_base = nil
    end
end

Hooks:PostHook(BowWeaponBase, "trigger_held", "Hawkeye_trigger_held", function(self, unit)
    if self._overcharged_enabled and managers.player:is_ability_cooldown_ended("HAWKEYE_Overcharge") then
        self._charge_start_t = self._charge_start_t or 0
        local delta_t = managers.player:player_timer():time() - self._charge_start_t
        local over_charge_multiplier = math.min(delta_t / self._overcharge_charge_min_duration, 1)
        if over_charge_multiplier >= 1 and not self._overcharged_shot then
            self._overcharged_shot = true
            HudChallengeNotification.queue("", "OVERCHARGED!")
        end
    end
end)
